"""


    AuthenticationToken


"""

from .authentication import Authentication


class AuthenticationToken(Authentication):

    def __init__(self, token):
        super().__init__()
        self._token = token

    def is_access(self, access_data):
        if self.token and self._token == access_data:
            return True
        return False

    @property
    def token(self):
        return self._token

    @property
    def md5hash(self):
        return Authentication.calc_md5hash(self._token)



"""

    Account class to provide basic functionality for convex network

"""
from typing import Any

from convex_api import Account as ConvexAPIAccount

from starfish.network.account_base import AccountBase


class ConvexAccount(ConvexAPIAccount, AccountBase):

    def sign_transaction(self, transaction: Any) -> Any:
        return self.sign(transaction)



"""


    AuthenticationBasic


"""

from .authentication import Authentication


class AuthenticationBasic(Authentication):

    def __init__(self, username, password):
        super().__init__()
        self._username = username
        self._password = password

    def is_access(self, access_data):
        if self._username and self._password and self._password == access_data:
            return True
        return False

    @property
    def username(self):
        return self._username

    @property
    def password(self):
        return self._password

    @property
    def md5hash(self):
        return Authentication.calc_md5hash(self._username)



"""

CryptoHash utils

"""
from typing import Any

from Crypto.Hash import SHA3_256


def hash_sha3_256(data: Any) -> str:
    if isinstance(data, str):
        data = data.encode('utf-8')

    messageDigest = SHA3_256.new()
    messageDigest.update(data)
    return messageDigest.hexdigest()





from starfish.agent.authentication.authentication import Authentication                              # noqa: F401
from starfish.agent.authentication.authentication_basic import AuthenticationBasic                   # noqa: F401
from starfish.agent.authentication.authentication_token import AuthenticationToken                   # noqa: F401



"""
    Inovke Asset
"""
import json
from typing import (
    Any,
    Generic
)

from starfish.asset.asset_base import AssetBase
from starfish.asset.data_asset import DataAsset
from starfish.types import TOperationAsset


class OperationAsset(DataAsset, Generic[TOperationAsset]):

    """

    Operation asset can be used to perform a invokable service

    :param metadata: Optional dictionary metadata to provide for the asset
        if non used then the class will generate a default metadata based on the data provided
    :type metadata: None or dict
    :param did: Optional did of the asset if it's registered
    :type did: None or str
    :param str, bytes data: Optional data to assign to this operation asset

    """
    def __init__(self, metadata_text: str, did: str = None, data: Any = None) -> None:
        DataAsset.__init__(self, metadata_text, did=did, data=data)

    @staticmethod
    def create(name: str, metadata: Any = None, data: Any = None, modes: Any = None) -> TOperationAsset:
        """

        Create a new OperationAsset.

        :param str name: Name of the asset
        :param dict metadata: Optional metadata to add to the assets metadata
        :param str, bytes data: Option data to pass to the operation asset
        :param list,tuple modes: Modes supported by this opreation, defaults to ['sync', 'async']

        :returns OperationAsset object

        """
        metadata = AssetBase.generateMetadata(name, 'operation', metadata)
        if 'operation' not in metadata:
            metadata['operation'] = {}
        if modes is None:
            modes = ['sync', 'async']
        metadata['modes'] = modes
        if not isinstance(modes, (tuple, list)):
            raise TypeError(f'modes {modes} must be a list or tuple of the operation modes supported')

        return OperationAsset(json.dumps(metadata), data=data)

    @staticmethod
    def create_orchestration(name: str, data: Any = None, metadata: Any = None, modes: Any = None) -> TOperationAsset:
        """

        Create a new OperationAsset with a class of orchestration.

        :param str name: Name of the asset
        :param str, bytes data: Option orchestration dict or json str to pass to the operation asset
        :param dict metadata: Optional metadata to add to the assets metadata
        :param list,tuple modes: Modes supported by this opreation, defaults to ['sync', 'async']

        :returns OperationAsset object

        """
        data_text = None
        if data:
            if isinstance(data, dict):
                data_text = json.dumps(data)
            elif isinstance(data, (str, bytes)):
                data_text = data
            else:
                raise TypeError('ochestration data can only be json str, bytes or a dict')

        metadata = AssetBase.generateMetadata(name, 'operation', metadata)
        if data_text:
            metadata = DataAsset.set_metadata_content_data(metadata, data=data_text, content_type='application/json')

        if 'operation' not in metadata:
            metadata['operation'] = {}
        metadata['operation']['class'] = 'orchestration'

        if modes is None:
            modes = ['sync', 'async']
        if not isinstance(modes, (tuple, list)):
            raise TypeError(f'modes {modes} must be a list or tuple of the operation modes supported')

        metadata['modes'] = modes

        return OperationAsset(json.dumps(metadata), data=data_text)

    def is_mode(self, mode_type: str) -> bool:
        """

        Check to see if this operation supports the mode provided.

        :param str mode_type: Mode type to check to see if this operation supports
        :return: Return True if this mode is supported
        :type: boolean

        """
        try:
            if 'operation' in self.metadata and self.metadata['operation']:
                return mode_type in self.metadata['operation'].get('modes', [])
        except ValueError:
            raise ValueError('Metadata does not contain operation->modes structure')

    @property
    def is_orchestration(self):
        """

        Return true if this operation asset is an orchestration
        """

        return self.class_name == 'orchestration'

    @property
    def class_name(self):
        """

        Return operation class type name , for the orchestration this will be 'orchestration'
        """
        if 'operation' in self.metadata and self.metadata['operation']:
            return self.metadata['operation'].get('class', None)



"""
    test_operation_asset

"""

import pytest
import secrets
import logging
import json

from starfish.asset import OperationAsset

OPERATION_METADATA = {
    'name': 'operation',
    'type': 'operation',
    'operation': {
        'modes': ['sync', 'async', 'test'],
    },
}


def test_init(metadata):
    asset = OperationAsset.create('new operation', OPERATION_METADATA)
    assert(asset)
    assert(isinstance(asset, OperationAsset))
    assert(asset.type_name == 'operation')


def test_operation_asset_create():
    asset_name = secrets.token_hex(8)

    asset = OperationAsset.create(asset_name)
    assert(asset)
    assert(asset.name == asset_name)

    values = {
        'name': 'value',
        'key1': secrets.randbelow(100000)
    }
    test_data = json.dumps(values)

    asset = OperationAsset.create(asset_name, metadata = {}, data = test_data)
    assert(asset)
    assert(asset.name == asset_name)
    assert(asset.type_name == 'operation')
    assert(asset.data == json.dumps(values).encode())
    assert(not asset.is_orchestration)


def test_operation_create_orchestration():
    asset_name = secrets.token_hex(8)

    values = {
        'name': 'value',
        'key1': secrets.randbelow(100000)
    }
    asset = OperationAsset.create_orchestration(asset_name, values)
    assert(asset)
    assert(asset.name == asset_name)
    assert(asset.type_name == 'operation')
    assert(asset.data == json.dumps(values).encode())
    assert(asset.is_orchestration)

    with pytest.raises(TypeError, match='ochestration data can only'):
        asset = OperationAsset.create_orchestration(asset_name, ['invalid', 'list'])


def test_operation_asset_mode(metadata, config):

    asset = OperationAsset.create('new operation', OPERATION_METADATA)
    for mode_name in OPERATION_METADATA['operation']['modes']:
        assert(asset.is_mode(mode_name))
        assert(not asset.is_mode(f'{mode_name}-badmode'))






"""

    Convex Contracts

"""

from starfish.network.convex.contract.contract_manager import ContractManager                               # noqa: F401
from starfish.network.convex.contract.did_contract import DIDContract                                       # noqa: F401
from starfish.network.convex.contract.provenance_contract import ProvenanceContract                         # noqa: F401



"""

    Job Module

"""


class JobStatus():
    SCHEDULE = 'scheduled'
    RUNNING = 'running'
    SUCCEEDED = 'succeeded'
    FAILED = 'failed'
    CANCELLED = 'cancelled'


JobStatusWorking = [
    JobStatus.SCHEDULE,
    JobStatus.RUNNING,
]

JobStatusDone = [
    JobStatus.SUCCEEDED,
    JobStatus.FAILED,
    JobStatus.CANCELLED
]

JobStatusFailed = [
    JobStatus.FAILED,
    JobStatus.CANCELLED
]

from starfish.job.job import Job                               # noqa: F401 E402



import json
import re
from typing import Any

from convex_api.utils import remove_0x_prefix

from starfish.asset.asset_base import AssetBase
from starfish.asset.bundle_asset import BundleAsset
from starfish.asset.data_asset import DataAsset
from starfish.asset.operation_asset import OperationAsset
from starfish.asset.provenance import Provenance
from starfish.asset.remote_data_asset import RemoteDataAsset        # noqa: F401


def create_asset(metadata: [str, dict, bytes], did: str = None, asset: AssetBase = None):
    """
    Create a new asset class based on the metadata. Once created assign
    the metadata to the asset and also the optional did

    :param str metadata_text: metadata text to test and create the correct asset object
    :param str did: optional did to assign to the asset
    :param TAssetBase asset: optional asset object to copy data (only for DataAsset)

    :return: Return an asset class based on the metadata type
    """

    metadata_text = None
    if isinstance(metadata, str):
        metadata_text = metadata
    elif isinstance(metadata, bytes):
        metadata_text = metadata.decode('utf-8')
    elif isinstance(metadata, dict):
        metadata_text = json.dumps(metadata)
    else:
        raise TypeError('metadata must be a string, bytes or dict')

    # now set the metadata for reading
    metadata = json.loads(metadata_text)
    asset_type = metadata.get('type', None)
    data = None
    if asset and hasattr(asset, 'data'):
        data = asset.data

    if asset_type == 'bundle':
        return BundleAsset(metadata_text, did=did)
    elif asset_type == 'operation':
        return OperationAsset(metadata_text, did=did, data=data)
    elif asset_type == 'dataset':
        return DataAsset(metadata_text, did=did, data=data)
    else:
        raise ValueError(f'Unknown asset type {asset_type}')
    return AssetBase(metadata_text, did=did)


def is_asset_hash_valid(asset_id, hash_hex):
    """

    :param str asset_id: asset id to check against
    :param str hash_hex: hex string to check
    :return: true if the hash string is the same as the asset_id
    :type: boolean
    """
    if asset_id or hash:
        return remove_0x_prefix(asset_id).lower() == remove_0x_prefix(hash_hex).lower()
    return False


def is_asset_id(text: str) -> bool:
    """
    Return true if the string is a valid asset string (64 hex chars)

    """
    return bool(re.fullmatch('(0x)?[0-9a-f]{64}', text, re.IGNORECASE))


def create_asset_provenance_publish(asset: AssetBase, agent_did: str = None):
    """

    Add a published provenance data to the asset metadata.
    Calling this method will make the asset 'new'. So the asset_id will change,
    and the asset.did will be set to None.

    :param str agent_did: DID of the agent that this asset will be registered with

    """
    metadata = asset.metadata
    provenance = Provenance(agent_did=agent_did)
    metadata['provenance'] = provenance.create_publish
    return create_asset(metadata, asset=asset)


def create_asset_provenance_invoke(asset: AssetBase, agent_did: str, job_id: str, asset_list: Any, inputs_text: str):
    """

    Add a invoke provenance data to the asset metadata.
    Calling this method will make the asset 'new'. So the asset_id will change,
    and the asset.did will be set to None.

    :param str agent_did: DID of the agent that this asset will be registered with

    """
    metadata = asset.metadata
    provenance = Provenance(agent_did=agent_did, activity_id=job_id, asset_list=asset_list, inputs_text=inputs_text)
    metadata['provenance'] = provenance.create_invoke
    return create_asset(metadata, asset=asset)



"""

    Network class to provide basic functionality for convex network

"""
from convex_api import (
    API,
    KeyPair
)

from starfish.network.convex.convex_account import ConvexAccount
from starfish.network.network_base import NetworkBase
from starfish.types import AccountAddress

DEFAULT_PACKAGE_NAME = 'starfish.network.convex.contract'

DID_CONTRACT_NAME = 'starfish.did'
PROVENANCE_CONTRACT_NAME = 'starfish.provenance'


class ConvexNetwork(NetworkBase):
    def __init__(self, url: str, contract_names=None) -> None:
        NetworkBase.__init__(self, url)
        self._convex = API(url)
        self._contract_names = contract_names or {}
        from starfish.network.convex.contract.contract_manager import ContractManager
        self._manager = ContractManager(self._convex, DEFAULT_PACKAGE_NAME)

    def create_account(self, key_pair: KeyPair = None) -> ConvexAccount:
        return self._convex.create_account(key_pair)

    def load_account(self, name, key_pair: KeyPair) -> ConvexAccount:
        return self._convex.setup_account(name, key_pair)

    def setup_account(self, name, key_pair: KeyPair) -> ConvexAccount:
        return self._convex.setup_account(name, key_pair)

    def register_account_name(self, name, account: ConvexAccount) -> ConvexAccount:
        return self._convex.setup_account(name, account)

    """

    Register DID with a DDO and resolve DID to a DDO

    """
    def register_did(self, account: ConvexAccount, did: str, ddo_text: str) -> bool:
        ddo_registry_contract = self._manager.load(
            'DIDContract',
            self._contract_names.get('DIDContract', DID_CONTRACT_NAME)
        )
        if ddo_registry_contract:
            return ddo_registry_contract.register_did(did, ddo_text, account)

    def resolve_did(self, did: str, account_address: AccountAddress = None) -> str:
        ddo_registry_contract = self._manager.load(
            'DIDContract',
            self._contract_names.get('DIDContract', DID_CONTRACT_NAME)
        )
        if account_address is None:
            account_address = ddo_registry_contract.address
        if ddo_registry_contract:
            return ddo_registry_contract.resolve(did,  account_address)
        return None

    """

    Register Provenance

    """
    def register_provenance(self, account: ConvexAccount, asset_id: str, data: str):
        provenance_contract = self._manager.load(
            'ProvenanceContract',
            self._contract_names.get('ProvenanceContract', PROVENANCE_CONTRACT_NAME)
        )
        result = provenance_contract.register_provenance(asset_id, data, account)
        return result

    def get_provenance_event_list(self, asset_id: str):
        provenance_contract = self._manager.load(
            'ProvenanceContract',
            self._contract_names.get('ProvenanceContract', PROVENANCE_CONTRACT_NAME)
        )
        return provenance_contract.event_list(asset_id, provenance_contract.address)

    @property
    def convex(self):
        return self._convex



"""

    Command Base Class

"""

from abc import (
    ABC,
    abstractmethod
)
from typing import Any

from convex_api.utils import is_address

from starfish.network.convex.convex_network import ConvexNetwork

DEFAULT_NETWORK_URL = 'https://convex.world'


class CommandBase(ABC):
    def __init__(self, name: str, sub_parser: Any = None) -> None:
        self._name = name
        self._sub_parser = sub_parser
        if sub_parser:
            self.create_parser(sub_parser)

    def is_command(self, name: str) -> bool:
        return self._name == name

    def get_network(self, url: str, contract_names: dict = None, default_url: str = None) -> Any:
        if url is None:
            url = default_url
        if url is None:
            url = DEFAULT_NETWORK_URL
        network = ConvexNetwork(url, contract_names=contract_names)
        return network

    def process_sub_command(self, args: Any, output: Any, command: str) -> None:
        is_found = False
        for command_item in self._command_list:
            if command_item.is_command(command):
                command_item.execute(args, output)
                is_found = True
                break

        if not is_found:
            self.print_help()

    def print_help(self) -> None:
        self._sub_parser.choices[self._name].print_help()

    def is_address(self, value):
        return is_address(value)

    @abstractmethod
    def create_parser(self, sub_parser: Any) -> Any:
        pass

    @abstractmethod
    def execute(self, args: Any, output: Any) -> None:
        pass

    @property
    def name(self) -> str:
        return self._name



"""
    starfish.provenance contract

"""

from convex_api.contract import Contract

from convex_api.utils import (
    add_0x_prefix,
    is_address,
    to_address
)

from starfish.network.convex.convex_account import ConvexAccount
from starfish.types import AccountAddress


class ProvenanceContract(Contract):

    def register_provenance(self, asset_id: str, data: str, account: ConvexAccount):
        quote_data = Contract.escape_string(data)
        command = f'(register {add_0x_prefix(asset_id)} "{quote_data}")'
        result = self.send(command, account)
        if result and 'value' in result:
            return {
                'timestamp': result['value']['timestamp'],
                'owner': to_address(result['value']['owner']),
            }
        return result

    def event_list(self, asset_id: str, account_address: AccountAddress):
        command = f'(event-list {add_0x_prefix(asset_id)})'
        if is_address(account_address):
            address = account_address
        else:
            address = account_address.address
        result = self.query(command, address)
        if result and 'value' in result:
            return ProvenanceContract.convert_event_list(result['value'])
        return result

    def owner_asset_list(self, account_address: AccountAddress):
        if is_address(account_address):
            address = account_address
        else:
            address = account_address.address
        command = f'(owner-list {address})'
        result = self.query(command, address)
        if result and 'value' in result:
            return result['value']
        return result

    @staticmethod
    def convert_event_list(items):
        event_list = []
        for item in items:
            event_list.append({
                'timestamp': item['timestamp'],
                'owner': to_address(item['owner']),
            })
        return event_list



"""

   Job class


"""

from typing import (
    Any,
    Generic
)

from starfish.job import (
    JobStatusDone,
    JobStatusWorking
)

from starfish.types import TJob


class Job(Generic[TJob]):
    """
        Create a Job object

        :param agent: agent object that created the listing.
        :type agent: :class:`.Agent` object to assign to this Listing
        :param str job_id: id of the job.
    """
    def __init__(self, job_id: str, status: str = None, outputs: Any = None, error: Any = None) -> None:
        """

        init the the Job Object Base with the agent instance

        :param int job_id: id of the job
        :param str status: status of the job
        :param dict outputs: dict or None for the outputs
        """
        self._job_id = job_id
        self._status = status
        self._outputs = outputs
        self._error = error
        super().__init__()

    @staticmethod
    def create_from_result(job_id: str, result: dict) -> TJob:
        if not isinstance(result, dict):
            raise TypeError('job result is not a dict')
        return Job(
            job_id,
            status=result.get('status', None),
            outputs=result.get('outputs', None),
            error=result.get('errror', None)
        )

    @property
    def is_done(self) -> bool:
        if self._status:
            return self._status in JobStatusDone
        return False

    @property
    def is_working(self) -> bool:
        if self._status:
            return self._status in JobStatusWorking
        return False

    @property
    def job_id(self) -> str:
        """
        Return the job id

        :return: Job id
        :type: int
        """
        return self._job_id

    @property
    def status(self) -> str:
        """
        Return the status for this job

        :return: status
        :type: str
        """
        return self._status

    @property
    def outputs(self) -> Any:
        """
        Return the ouptuts for this job

        :return: results
        :type: dict or None
        """
        return self._outputs

    @property
    def error(self) -> Any:
        """
        Return the ouptuts for this job

        :return: results
        :type: dict or None
        """
        return self._error

    @property
    def result(self):
        """

        Return the job result of a completed job. If the job is not completed then
        the 'status' will contain the current job status.

        :param str job_id: Job id of the job to get the status.
        :return: Job status
        :type: str

        """
        result = {
            'status': self._status,
        }
        if self._outputs:
            result['outputs'] = self._outputs
        if self._error:
            result['error'] = self._error
        return result

    def __str__(self) -> str:
        text = 'job:'
        if self._status:
            text += f'{self._status}'
        if self._outputs:
            text += f' = {self._outputs}'
        return text



"""


    Authentication

"""
import hashlib

from abc import (
    ABC,
    abstractmethod,
    abstractproperty
)


class Authentication(ABC):

    @abstractmethod
    def is_access(self, access_data):
        return False

    @abstractproperty
    def md5hash(self):
        return None

    @staticmethod
    def calc_md5hash(text):
        return hashlib.md5(text.encode()).hexdigest()



"""

    Data Asset

"""

import csv
import io
import json as json_module
import os
from mimetypes import MimeTypes
from typing import (
    Any,
    Generic,
    List
)

from starfish.asset.asset_base import AssetBase
from starfish.types import TDataAsset
from starfish.utils.crypto_hash import hash_sha3_256


class DataAsset(AssetBase, Generic[TDataAsset]):
    """

    File asset can be used manage a data asset on the dex network

    :param metadata: Dictionary metadata to provide for the asset.
    :type metadata: None or dict
    :param did: Optional did of the asset if it's registered
    :type did: None or str
    :param str data: Optional data of the asset, this can be str or bytes

    """
    def __init__(self, metadata_text: str, did: str = None, data: Any = None) -> None:

        AssetBase.__init__(self, metadata_text, did)
        if data:
            if isinstance(data, (str, bytes)):
                if isinstance(data, str):
                    data = data.encode('utf-8')
            else:
                raise TypeError('data can only be str or bytes')

        self._data = data

    @staticmethod
    def create(
        name: str,
        data: Any = None,
        metadata: Any = None,
        json: Any = None,
        csv_rows: List[Any] = None,
        csv_header: List[str] = None
    ) -> TDataAsset:
        """

        Create a new DataAsset using string or bytes data.

        :param str name: Name of the asset to create
        :param str, bytes data: Data to assign to the asset
        :param dict metadata: Optional metadata to add to the assets metadata

        :return: a new DataAsset
        :type: :class:`.DataAsset`

        """
        content_type = None
        if json:
            data = json_module.dumps(json)
            content_type = 'application/json'
        elif csv_rows:
            csv_output = io.StringIO()
            writer = csv.DictWriter(csv_output, fieldnames=csv_header)
            writer.writeheader()
            for row in csv_rows:
                writer.writerow(row)
            data = csv_output.getvalue()
            content_type = 'text/csv'

        metadata = AssetBase.generateMetadata(name, 'dataset', metadata)
        metadata = DataAsset.set_metadata_content_data(metadata, data=data, content_type=content_type)

        return DataAsset(json_module.dumps(metadata), data=data)

    @staticmethod
    def create_from_file(name: str, filename: str, metadata: Any = None, did: str = None, is_read: bool = True) -> TDataAsset:
        """

        Create a new DataAsset using a file or filename.

        :param str name: Name of the asset to create
        :param str filename: If the filename is assigned to a valid file,
            the contents will be saved in the asset
        :param dict metadata: Optional metadata to add to the assets metadata
        :param str did: Option DID to assign to this asset
        :param bool is_read: If True read the file contents in as asset data.

        :return: a new DataAsset
        :type: :class:`.DataAsset`

        """

        metadata = AssetBase.generateMetadata(name, 'dataset', metadata)
        if 'filename' not in metadata:
            metadata['filename'] = os.path.basename(str(filename))
        data = None
        if os.path.exists(filename):
            content_type = 'application/octet-stream'
            mime = MimeTypes()
            mime_type = mime.guess_type(f'file://{filename}')
            if mime_type:
                content_type = mime_type[0]
            if is_read:
                with open(filename, 'rb') as fp:
                    data = fp.read()
            metadata = DataAsset.set_metadata_content_data(metadata, data=data, content_type=content_type)

        return DataAsset(json_module.dumps(metadata), data=data)

    @staticmethod
    def set_metadata_content_data(metadata: dict, data: Any, content_type: str = None) -> dict:
        """

        Static method to generate the metadata values associated with storing data with an asset.

        This method will try to assign 'contentType', 'contentHash', 'contentLength' in the metadata dict

        :param dict metadata: Metadata to assign and return
        :param str,bytes data: Data to get the hash and length from
        :param str content_type: Optional content_type, if not provided the data will be used to determin the contentType

        :returns: a modified metadata dict with the new fields set

        """
        if not isinstance(metadata, dict):
            raise TypeError('you must assign a dict as the metadata to set the content data metadata')

        if content_type is None:
            content_type = 'application/octet-stream'
            if isinstance(data, str):
                content_type = 'text/plain'
            elif isinstance(data, bytes):
                content_type = 'application/octet-stream'

            if data:
                # test for json
                try:
                    json_data = json_module.loads(data)
                    if json_data:
                        content_type = 'application/json'
                except ValueError:
                    pass

        if 'contentType' not in metadata and content_type:
            metadata['contentType'] = content_type
        if data:
            if 'contentHash' not in metadata:
                metadata['contentHash'] = hash_sha3_256(data)
            if 'contentLength' not in metadata:
                metadata['contentLength'] = len(data)
        return metadata

    def save_to_file(self, filename: str) -> None:
        """
        Saves the data in the data asset to a file.

        :param str filename: Filename to save the data.

        """

        if self._data:
            with open(filename, 'wb') as fp:
                fp.write(self._data)

    @property
    def data(self) -> Any:
        return self._data

    @property
    def is_json(self) -> bool:
        return self.metadata['contentType'] == 'application/json'

    def json(self) -> Any:
        return json_module.loads(self._data)

    @property
    def is_csv(self) -> bool:
        return self.metadata['contentType'] == 'text/csv'

    def csv(self) -> Any:
        stream = io.StringIO(self._data.decode('utf-8'))
        reader = csv.DictReader(stream)
        rows = []

        for row in reader:
            rows.append(row)
        return rows



"""

    Tool command Account Create


"""
import logging
import os
from typing import Any

from convex_api import KeyPair

from .command_base import CommandBase

logger = logging.getLogger(__name__)


class AccountCreateCommand(CommandBase):

    def __init__(self, sub_parser: Any = None) -> None:
        super().__init__('create', sub_parser)

    def create_parser(self, sub_parser):
        parser = sub_parser.add_parser(
            self._name,
            description='Create a new account',
            help='Create a new account'

        )
        parser.add_argument(
            'password',
            help='Password of new account'
        )

        parser.add_argument(
            'keyfile',
            nargs='?',
            help='Optional name of the keyfile to write the account key data to'
        )
        return parser

    def execute(self, args: Any, output: Any) -> None:
        network = self.get_network(args.url)
        key_pair = None
        if args.password and args.keyfile:
            key_pair = KeyPair.import_from_file(args.keyfile, args.password)
        else:
            key_pair = KeyPair()
        account = network.create_account(key_pair)
        logger.debug(f'create new account {account.address}')
        if args.keyfile and not os.path.exists(args.keyfile):
            logger.debug(f'writing key file to {args.keyfile}')
            account.key_pair.export_to_file(args.keyfile, args.password)
        else:
            logger.debug('writing key file to ouptut')
            output.add_line(account.key_pair.export_to_text(args.password))
        output.add_line(account.address)
        output.set_value('public_key', account.public_key)
        output.set_value('export_key', account.key_pair.export_to_text(args.password))
        output.set_value('address', account.address)
